/**
* @file  hardware_info_items.h
* @brief Schnittstelle fuer die Hilfsklassen zu FileHardwareInfo.
*/
//////////////////////////////////////////////////////////////////////
#pragma once
#ifndef HARDWARE_INFO_ITEMS_H
#define HARDWARE_INFO_ITEMS_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif
// Strukturdefinitionen
#include "file/interface/hardware_info_structs.h"

// Verwaltung der Strukturen in Baum.
#include <string>
#include <vector>
#include <array>
#include <set>
#include <list>

enum EnumItemHardwareInfoType
{
    Root = -1,
    MainBoard = 0,
    Module,
    ModuleWithPlaces,
    SubModule
};

// ########################################################
// Hauptkommunikation

enum EnumModuleRS232Communication
{
    // MasterIV - Geräte besitzen immer einen SerialPort
    SerialPort = 16 // M016 Serial Port SubD
};
typedef std::set<EnumModuleRS232Communication> setModuleRS232Communication;
static const setModuleRS232Communication moduleRS232Communication_all
{
    SerialPort
};

enum EnumModuleRS485Communication
{
    RS485Old = 100000, // RS485
    RS485New = 35      // M035 RS485 Com. Port
};
typedef std::set<EnumModuleRS485Communication> setModuleRS485Communication;
static const setModuleRS485Communication moduleRS485Communication_all
{
    RS485Old,
    RS485New
};

enum EnumOnBoardRS485Communication
{
    RS485_KYO_Oneloc     = 50011,  // RS485 auf dem Mainboard gilt für KYO Oneloc
    RS485_EVO35Universal = 50019   // RS485 auf dem Mainboard gilt für EVO 3.5 Universal
}; typedef std::set<EnumOnBoardRS485Communication> setOnBoardRS485Communication;
static const setOnBoardRS485Communication onBoardRS485Communication_all
{
    RS485_KYO_Oneloc,
    RS485_EVO35Universal
};

enum EnumModuleUSBCommunication
{
    USBOld = 100004 // USB
    // EVO-Line - Geräte besitzen immer USB.
};
typedef std::set<EnumModuleUSBCommunication> setModuleUSBCommunication;
static const setModuleUSBCommunication moduleUSBCommunication_all
{
    USBOld
};

enum EnumModuleLANCommunication
{
    LANOld                     = 100001, // TCPIP-XPort
    LANNew                     = 11      // M011 Ethernet Port
};
typedef std::set<EnumModuleLANCommunication> setModuleLANCommunication;
static const setModuleLANCommunication moduleLANCommunication_all
{
    LANOld,
    LANNew
};

enum EnumOnBoardLANCommunication
{
    LAN_KYO_Oneloc              = 50011,  // LAN auf dem Mainboard gilt für KYO Oneloc
    LAN_EVO35Universal          = 50019,  // LAN auf dem Mainboard gilt für EVO 3.5 Universal
    LAN_KYO_Cenloc              = 50020   // LAN auf dem Mainboard gilt für KYO Cenloc
};typedef std::set<EnumOnBoardLANCommunication> setOnBoardLANCommunication;
static const setOnBoardLANCommunication onBoardLANCommunication_all
{
    LAN_KYO_Oneloc,
    LAN_EVO35Universal,
    LAN_KYO_Cenloc
};

enum EnumModuleWLANCommunication
{
    WLANOld             = 100002, // WLAN-MatchPort
    WLAN_RS9110         = 1,      // WLAN RS9110-N-11-24 Auflötmodul
    WLAN_RS9110_Plug    = 105002, // WLAN RS9110-N-11-24 Ansteckmodul
    WLAN_TI_CC3135      = 95,     // WLAN TI CC3135 ML01 Auflötmodul
    WLAN_TI_CC3135_Plug = 105007  // WLAN TI CC3135 MS03 Ansteckmodul
};
typedef std::set<EnumModuleWLANCommunication> setModuleWLANCommunication;
static const setModuleWLANCommunication moduleWLANCommunication_all
{
    WLANOld,
    WLAN_RS9110,
    WLAN_RS9110_Plug,
    WLAN_TI_CC3135,
    WLAN_TI_CC3135_Plug
};

enum EnumModuleMobileCommunication
{
    MC35_39 = 100003, // Mobile MC35-39
    MC55    = 105001, // Mobile MC55i
                      // Sonderfall: Wenn das Auflötmodul M015 bestückt wurde und kein Ansteckmodul angegeben ist, wird von einem MC55 augegangen.
    BG96    = 105003, // Mobile BG96 Quectel
    EG95    = 105004  // Mobile EG95 Quectel
};
typedef std::set<EnumModuleMobileCommunication> setModuleMobileCommunication;
static const setModuleMobileCommunication moduleMobileCommunication_all
{
    MC35_39,
    MC55,
    BG96,
    EG95
};

enum EnumModuleBluetoothCommunication
{
    Bluegiga = 100005 // Bluegiga Wrap Thor
};
typedef std::set<EnumModuleBluetoothCommunication> setModuleBluetoothCommunication;
static const setModuleBluetoothCommunication moduleBluetoothCommunication_all
{
    Bluegiga
};

enum EnumModuleMMCCommunication
{
    MMCRemovable = 100009 // Removable media device
};
typedef std::set<EnumModuleMMCCommunication> setModuleMMCCommunication;
static const setModuleMMCCommunication moduleMMCCommunication_all
{
    MMCRemovable
};

enum EnumModuleUSBHostCommunication
{
    JackPlug = 24,     // M024 USB 2.0 A Jack-Plug
    Vinculum = 100010  // USB-Host Vinculum
};
typedef std::set<EnumModuleUSBHostCommunication> setModuleUSBHostCommunication;
static const setModuleUSBHostCommunication moduleUSBHostCommunication_all
{
    JackPlug,
    Vinculum
};

// ########################################################
// Zusatzbaugruppen

enum EnumModuleAccessControl
{
    AccessControlOld                   = 100006,    // RS485 für ZK.
    AccessControlNew                   = 14         // M014 RS485 + 12V Supply
};
typedef std::set<EnumModuleAccessControl> setModuleAccessControl;
static const setModuleAccessControl moduleAccessControl_all
{
    AccessControlOld,
    AccessControlNew
};

enum EnumOnBoardAccessControl
{
    AccessControl_KYO_Oneloc       = 50011,     // ZK auf Mainboard gilt fuer KYO Oneloc
    AccessControl_EVO_35_Universal = 50019,     // ZK auf Mainboard gilt fuer EVO 3.5 Universal
    AccessControl_KYO_Cenloc       = 50020      // ZK auf Mainboard gilt fuer KYO Cenloc
};
typedef std::set<EnumOnBoardAccessControl> setOnBoardAccessControl;
static const setOnBoardAccessControl onBoardAccessControl_all
{
    AccessControl_KYO_Oneloc,
    AccessControl_EVO_35_Universal,
    AccessControl_KYO_Cenloc
};

enum EnumModuleBiokey
{
    Biokey3000 = 104000, // Biokey 3000
    Biokey4000 = 104001, // Biokey 4000
    Biokey4020 = 104002, // Biokey 4020
    Saturn01   = 104003  // Saturn01
};
typedef std::set<EnumModuleBiokey> setModuleBiokey;
static const setModuleBiokey moduleBiokey_all
{
    Biokey3000,
    Biokey4000,
    Biokey4020,
    Saturn01
};


enum EnumModuleGPS
{
    Mod01_007 = 7,      // GPS_Mod01_007
    Furuno    = 100011, // Furuno GN-80
    Lea5H     = 100012, // Lea5H
    Navimouse = 100013  // Navimouse mit Lea5H
};
typedef std::set<EnumModuleGPS> setModuleGPS;
static const setModuleGPS moduleGPS_all
{
    Mod01_007,
    Furuno,
    Lea5H,
    Navimouse
};

enum EnumModuleDisplay
{
    Monocrome_3_5       = 110001, // Monocrome Display 3,5"
                                  // Sonderfall: Wenn das Auflötmodul M018 bestückt wurde und kein Ansteckmodul angegeben ist, wird von einem Monocrome_3_5 augegangen.
    Monocrome_3_5_Large = 100007, // Monocrome Display 3,5" gross fuer BDE-MasterIV.
    Color_1_9           = 110002, // Color TFT 1,9" 220x176
    Color_2_8           = 110003, // Color TFT 2,8" 320x240
                                  // Sonderfall: Wenn das Auflötmodul M030 bestückt wurde und kein Ansteckmodul angegeben ist, wird von einem Color_2_8 augegangen.
    Color_4_3           = 110004, // Color TFT 4,3" 420x272
                                  // Sonderfall: Wenn das Auflötmodul M019 bestückt wurde und kein Ansteckmodul angegeben ist, wird von einem Color_4_3 augegangen.
    Color_2_0           = 110005, // Color TFT 2,0" 220x176
    Color_3_2           = 110006, // Color TFT 3.2" 320x240
    Color_3_8           = 110007, // Color TFT 3.8" 320x240
    Color_3_5           = 110008, // Color TFT 3.5" 320x480
    Color_7             = 110009, // Color TFT 7" 800x480
    Color_V2_3_5        = 110010, // Color IPS 3.5" 320x480
    Color_5_0           = 110011  // Color TFT 5.0" 800x480
};
typedef std::set<EnumModuleDisplay> setModuleDisplay;
static const setModuleDisplay moduleDisplay_all
{
    Monocrome_3_5,
    Monocrome_3_5_Large,
    Color_1_9,
    Color_2_8,
    Color_4_3,
    Color_2_0,
    Color_3_2,
    Color_3_8,
    Color_3_5,
    Color_7,
    Color_V2_3_5,
    Color_5_0
};

enum EnumModuleTouch
{
    Touch_Evo_4_3            = 110101, // Touchscreen + SMT-C-T10I
                                       // Sonderfall: Wenn das Auflötmodul M019 bestückt wurde und kein Ansteckmodul angegeben ist, wird von einem Touch_Evo_4_3 augegangen.
    Touch_Evo_2_8            = 110102, // Resistive Touch 3.5"
                                       // Sonderfall: Wenn das Auflötmodul M030 bestückt wurde und kein Ansteckmodul angegeben ist, wird von einem Touch_Evo_2_8 augegangen.
    Touch_V2_Evo_4_3         = 110103, // Neue Version des Touchkontoroller für EVO_4_3 
	Touch_Evo_4_3_Sis        = 110104, // EVO 4.3 Sis-Touch neue Version
    Touch_Evo_Intera_II      = 110105, // FT6X36_2, 74"
    Touch_V1_Evo_5_0_Pure    = 110106, // Variante: Touch FT7311 5"
    Touch_V2_Evo_5_0_Pure    = 110107, // Variante: Touch ILI2511 5"
    Touch_V3_Evo_5_0_Pure    = 110108  // Variante: Touch GT911 5"
};
typedef std::set<EnumModuleTouch> setModuleTouch;
static const setModuleTouch moduleTouch_all
{
    Touch_Evo_4_3,
    Touch_Evo_2_8,
    Touch_V2_Evo_4_3,
    Touch_Evo_4_3_Sis,
    Touch_V1_Evo_5_0_Pure,
    Touch_V2_Evo_5_0_Pure,
    Touch_V3_Evo_5_0_Pure
};

enum EnumModuleTemperatureSensor
{
    TempSens_MLX90614xCF = 99      // M099 Body Temp Sensor
};
typedef std::set<EnumModuleTemperatureSensor> setModuleTemperatureSensor;
static const setModuleTemperatureSensor moduleTemperatureSensor_all
{
    TempSens_MLX90614xCF
};

class ItemBoard;

typedef std::list<ItemBoard *> listItemBoard;
typedef std::set<uint32_t> setModuleId;

class ItemHardwareInfoBase
{
public:
    ItemHardwareInfoBase( ItemHardwareInfoBase *parent, EnumItemHardwareInfoType typeId );
    virtual ~ItemHardwareInfoBase();

    ItemHardwareInfoBase *parent();
    void setParent( ItemHardwareInfoBase *parent );

    EnumItemHardwareInfoType typeId();
    void setTypeId( EnumItemHardwareInfoType typeId );

private:
    ItemHardwareInfoBase *m_parent;
    EnumItemHardwareInfoType m_typeId;

public:
    void setAutoCorrectModuleDisabled( bool state );
    bool autoCorrectModuleDisabled();

private:
    static bool m_autoCorrectModuleDisabled;
};

class ItemJumper
{
public:
    ItemJumper();
    virtual ~ItemJumper();

public:
    void reset();
    bool writeSerialize( std::vector<uint8_t>::iterator iter );
    bool readSerialize( std::vector<uint8_t>::const_iterator iter );

    HARDWARECONFIGJUMPER &hcj();

protected:
    std::array<uint8_t, sizeof( HARDWARECONFIGJUMPER )> m_structData;

public:
    std::string toJson();
};

class ItemBoard : public ItemHardwareInfoBase
{
public:
    ItemBoard( ItemHardwareInfoBase *parent = 0, EnumItemHardwareInfoType typeId = MainBoard );
    virtual ~ItemBoard();

public:
    // Tabelle der Id's, Stand: 03.02.2014
    // Modul Typ 0 Hauptplatinen Nummernkreis 50.000 bis 99.999
    // Modul Typ 1 und 2  Auflötmodule im Datafox Standard Raster Nummernkreis 1 bis 49.999
    // Modul Typ 3 Ansteckmodule für Modulplatinen Typ 0 und 2 Nummernkreis ab 100.000
    enum BoardIdGroup
    {
        Standard                    = 1,
        StandardEnd                 = 49999,
        Hauptplatine                = 50000,
        HauptplatineEnd             = 99999,
        Tastatur                    = 101000,
        TastaturEnd                 = 101999,
        Transponderleser            = 102000,
        TransponderleserEnd         = 102999,
        Barcode                     = 103000,
        BarcodeEnd                  = 103999,
        Fingerprint                 = 104000,
        FingerprintEnd              = 104999,
        Kommunikation               = 105000,
        KommunikationEnd            = 105999,
        Leuchtflaechen              = 106000,
        LeuchtflaechenEnd           = 106999,
        Displays                    = 110000,
        DisplaysEnd                 = 110099,
        Touch                       = 110100,
        TouchEnd                    = 110199,
    };

    // Tabelle der Id's, Stand: 01.12.2014
    enum BoardId
    {
        // Hauptplatinen
        MainboardIPCExtensionA                             = 50003, // IPC Vario 5.7 Mainboard
        MainboardIPCStandard                               = 50005, // IPC Standard Mainboard
        MainboardIOBoxTerminal                             = 50006, // IO-Box Terminal
        MainboardCombiV4                                   = 50007, // Kombi V4 Mainboard
        MainboardDockingstation                            = 50008, // Dockingstation Mainboard
        MainboardIOBox                                     = 50009, // IO-Box Top Hat Rail
        MainboardQ7                                        = 50010, // Motherboard Q7
        Mainboard_KYO_Oneloc                               = 50011, // KYO Oneloc Mainboard
        MainboardCombiV1_3                                 = 50014, // Kombi V1-3 Mainboard
        MainboardBDEMasterIV                               = 50015, // BDE-MasterIV Mainboard
        MainboardTimeboy4                                  = 50016, // Timeboy-V4 Mainboard
        MainboardFahrzeugdatenlogger                       = 50017, // Vehicle Logger Mainboard
        MainBoard_EVO_35_Universal                         = 50019, // EVO 3.5 Universal Mainboard
        MainBoard_KYO_Cenloc                               = 50020, // KYO Cenloc Mainboard 

        // Auflötmodule
        ModuleWLAN                                         =  1, // M001 WLAN Redpine RS9110
        ModuleChargeRegulator                              =  4, // M004 Charge Regulator
        ModuleDO                                           =  5, // M005 Relay Output
        ModuleDI                                           =  6, // M006 Digital Input
        ModuleGPS                                          =  7, // M007 GPS LEA6-H
        ModuleAI                                           =  8, // M008 Analog Input
        Module009                                          =  9, // M009 Mobil Master Port
        Module010                                          = 10, // M010 Backlight Control
        Module011                                          = 11, // M011 Ethernet Port
        ModuleDIDO                                         = 12, // M012 Digital In-/Output
        Module013                                          = 13, // M013 Position Receiver
        Module014                                          = 14, // M014 RS485 + 12V Supply
        Module015                                          = 15, // M015 Mobile Phone Plug
        Module016                                          = 16, // M016 Serial Port SubD
        Module017                                          = 17, // M017 Micro SD Card
        Module018                                          = 18, // M018 Grafic Adapter
        Module019                                          = 19, // M019 Grafic Adapter
        Module020                                          = 20, // M020 Touch Key
        Module021                                          = 21, // M021 Double Serial Port
        Module024                                          = 24, // M024 USB 2.0 A Jack-Plug
        Module029                                          = 29, // M029 POE Supply
        Module030                                          = 30, // M030 Grafic Adapter
        Module032                                          = 32, // M032 Serial Port M.-DIN
        Module035                                          = 35, // M035 RS485 Com. Port
        Module037                                          = 37, // M037 Single Serial Port
        Module038                                          = 38, // M038 PoE KYO Oneloc
        Module042                                          = 42, // M042 Terminalmodul ZKK
        Module043                                          = 43, // M043 Bluetooth
        Module046                                          = 46, // M046 WLAN Mobil Modul
        Module047                                          = 47, // M047 Transpondermodul
        Module048                                          = 48, // M048 Adapter Modul01
        Module050                                          = 50, // M050 Barcode integriert
        Module051                                          = 51, // M051_Digital_Out_PP 12V
        Module052                                          = 52, // M052 RS232 Com. Port
        Module053                                          = 53, // M053 LED 4x
        Module057                                          = 57, // M057 Graphic Adapter
        Module062                                          = 62, // M062 Monitored Input 4x
        Module064                                          = 64, // M064 UART internal
        Module065                                          = 65, // M065 ZK Kombi Mon In
        Module077                                          = 77, // M077_ZKK_Wiegand_Conv
        Module078                                          = 78, // M078 SRAM
        Module081                                          = 81, // M081 POE Supply V2
        Module083                                          = 83, // M083 RS485 UnZ
        Module085                                          = 85, // M085_Naeherungssensor43
        Module086                                          = 86, // M086 Graphic Adapter 7"
        Module088                                          = 88, // M088 Analog Input 0-20mA
        Module091                                          = 91, // M091_Naeherungssensor
        Module099                                          = 99, // M099 Body Temp Sensor
        Module105                                          = 105, // M105_Digital_Out_PP GND (wie M051 nur gegen GND geschaltet)

        ModuleOldAI                                        = 100016,
        ModuleOldDI                                        = 100017,
        ModuleOldDO                                        = 100018

    };
public:
    void reset();
    bool writeSerialize( std::vector<uint8_t> &data );
    bool readSerialize( std::vector<uint8_t>::const_iterator iterRead, const std::vector<uint8_t> &data );

    HARDWARECONFIG &hc();

    typedef std::vector<ItemBoard> typeChilds;
    typedef std::vector<ItemJumper> typeJumpers;

public:
    listItemBoard collectModuleIds( const setModuleId &ids );
    static std::string idGroup( uint32_t id );

protected:
    std::array<uint8_t, sizeof( HARDWARECONFIG )> m_structData;
    typeChilds m_childs;
    typeJumpers m_jumpers;
    void getDefault( ItemBoard &item );

public:
    std::string toJson();
    typeChilds &childs();
    typeJumpers &jumpers();
};

class ItemDeviceHardwareInfo : public ItemHardwareInfoBase
{
public:
    ItemDeviceHardwareInfo( ItemHardwareInfoBase *parent = 0, EnumItemHardwareInfoType typeId = Root );
    virtual ~ItemDeviceHardwareInfo();

public:
    void reset();
    bool writeSerialize( std::vector<uint8_t> &buffer );
    bool readSerialize( const std::vector<uint8_t> &buffer );

    HARDWARECONFIGNODE &hcn();

public:
    listItemBoard collectModuleIds( const setModuleId &ids );

    listItemBoard moduleRS232Communication( const setModuleRS232Communication &module = moduleRS232Communication_all );
    listItemBoard moduleRS485Communication( const setModuleRS485Communication &module = moduleRS485Communication_all );
    listItemBoard onBoardRS485Communication( const setOnBoardRS485Communication &module = onBoardRS485Communication_all );
    listItemBoard moduleUSBCommunication( const setModuleUSBCommunication &module = moduleUSBCommunication_all );
    listItemBoard moduleLANCommunication( const setModuleLANCommunication &module = moduleLANCommunication_all );
    listItemBoard onBoardLANCommunication( const setOnBoardLANCommunication &boards = onBoardLANCommunication_all );
    listItemBoard moduleWLANCommunication( const setModuleWLANCommunication &module = moduleWLANCommunication_all );
    listItemBoard moduleMobileCommunication( const setModuleMobileCommunication &module = moduleMobileCommunication_all );
    listItemBoard moduleBluetoothCommunication( const setModuleBluetoothCommunication &module = moduleBluetoothCommunication_all );
    listItemBoard moduleMMCCommunication( const setModuleMMCCommunication &module = moduleMMCCommunication_all );
    listItemBoard moduleUSBHostCommunication( const setModuleUSBHostCommunication &module = moduleUSBHostCommunication_all );

    listItemBoard moduleAccessControl( const setModuleAccessControl &module = moduleAccessControl_all );
    listItemBoard onBoardAccessControl( const setOnBoardAccessControl &boards = onBoardAccessControl_all );
    listItemBoard moduleBiokey( const setModuleBiokey &module = moduleBiokey_all );
    listItemBoard moduleGPS( const setModuleGPS &module = moduleGPS_all );

    listItemBoard moduleDisplay( const setModuleDisplay &module = moduleDisplay_all );
    listItemBoard moduleTouch( const setModuleTouch &module = moduleTouch_all );
    listItemBoard moduleTemperatureSensor( const setModuleTemperatureSensor &module = moduleTemperatureSensor_all );

protected:
    std::array<uint8_t, sizeof( HARDWARECONFIGNODE )> m_structData;
    ItemBoard::typeChilds m_childs;

public:
    std::string toJson();
    ItemBoard::typeChilds &childs();
};

#endif
