/**
* @file  mcu_memory.cpp
* @brief Implementierung der Klasse MCUMemory.
*/
//////////////////////////////////////////////////////////////////////
#include "file/interface/mcu_memory.h"
#include <memory.h>
#if !defined(_MSC_VER) && !defined(__MINGW32__)
#include <string.h>
#endif


MCUMemory::MCUMemory()
{
    // Speicher reservieren
#if MCU_SIZE_EEPROM
    m_eeprom = new unsigned char[MCU_SIZE_EEPROM];
    m_eepromUse = new unsigned int[MCU_SIZE_EEPROM / 32];
#endif
    m_program = new unsigned char[MCU_SIZE_PROGRAM];
    m_programUse = new unsigned int[MCU_SIZE_PROGRAM / 32];

    // Initialisierung
    Reset();
}

MCUMemory::~MCUMemory()
{
    // Speicher freigeben
#if MCU_SIZE_EEPROM
    if ( m_eeprom )
    {
        delete[] m_eeprom;
    }
    m_eeprom = nullptr;
    if ( m_eepromUse )
    {
        delete[] m_eepromUse;
    }
    m_eepromUse = nullptr;
#endif
    if ( m_program )
    {
        delete[] m_program;
    }
    m_program = nullptr;
    if ( m_programUse )
    {
        delete[] m_programUse;
    }
    m_programUse = nullptr;
}

void MCUMemory::Reset()
{
    // Speicher initialisieren
#if MCU_SIZE_EEPROM
    memset(m_eeprom, 0xFF, MCU_SIZE_EEPROM);
    memset(m_eepromUse, 0, sizeof(unsigned int)*(MCU_SIZE_EEPROM / 32));
#endif
    memset(m_program, 0xFF, MCU_SIZE_PROGRAM);
    memset(m_programUse, 0,  sizeof(unsigned int)*(MCU_SIZE_PROGRAM / 32));
}

#if MCU_SIZE_EEPROM
void MCUMemory::SetEepromUsebit(unsigned int address)
{
    m_eepromUse[address / 32] |= (1 << (address % 32));
}

unsigned char MCUMemory::GetEepromUsebit(unsigned int address)
{
    return (m_eepromUse[address / 32] & (1 << (address % 32))) ? 1 : 0;
}

unsigned int MCUMemory::GetEepromFirstAddress()
{
    unsigned int address, firstAddress = 0;
    for (address=0; address < MCU_SIZE_EEPROM; address++) {

        if (GetEepromUsebit(address)) {

            firstAddress = address;
            break;
        }
    }

    return firstAddress;
}

unsigned int MCUMemory::GetEepromLastAddress()
{
    unsigned int address, lastAddress = 0;
    for (address=0; address < MCU_SIZE_EEPROM; address++) {

        if (GetEepromUsebit(address)) {

            lastAddress = address;
        }
    }

    return lastAddress;
}
#endif

void MCUMemory::SetProgramUsebit(unsigned int address)
{
    m_programUse[address / 32] |= (1 << (address % 32));
}

unsigned char MCUMemory::GetProgramUsebit(unsigned int address)
{
    return (m_programUse[address / 32] & (1 << (address % 32))) ? 1 : 0;
}

unsigned int MCUMemory::GetProgramFirstAddress()
{
    unsigned int address, firstAddress = 0;
    for (address=0; address < MCU_SIZE_PROGRAM; address++) {

        if (GetProgramUsebit(address)) {

            firstAddress = address;
            break;
        }
    }

    return firstAddress;
}

unsigned int MCUMemory::GetProgramLastAddress()
{
    unsigned int address, lastAddress = 0;
    for (address=0; address < MCU_SIZE_PROGRAM; address++) {

        if (GetProgramUsebit(address)) {

            lastAddress = address;
        }
    }

    return lastAddress;
}

unsigned int MCUMemory::GetProgramSize()
{
    return GetProgramLastAddress() - GetProgramFirstAddress() + 1;
}

